﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ShapeVolume;
using SharpWorld;
using Variables;

namespace EntitySpawn
{
    public interface LifetimeParams : WsItemParams
    {
        /// <summary>
        /// 诞生时间点
        /// </summary>
        UInt128 SpawnStart { get; set; }
        /// <summary>
        /// 寿命长度累计
        /// </summary>
        UInt128 LifetimeAcc { get; set; }
        /// <summary>
        /// 成熟体最小质量
        /// </summary>
        double MatureVolume { get; set; }

        /// <summary>
        /// 成长期长度
        /// </summary>
        VariableULong GrowthTimeCount { get; }
        /// <summary>
        /// 壮年期长度
        /// </summary>
        VariableULong MatureTimeCount { get; }
        /// <summary>
        /// 死亡时间点
        /// </summary>
        VariableULong DeathTimePoint { get; }

        /// <summary>
        /// 繁育周期
        /// </summary>
        VariableULong SpawnPeriods { get; }
        /// <summary>
        /// 繁育次数
        /// </summary>
        UInt32 SpawnTimes { get; set; }
    }

    public class LifetimeSystem : WsSystem<LifetimeParams>, WsExecutableSystem
    {
        private Dictionary<UInt128, LifetimeParams> _items_store = new Dictionary<UInt128, LifetimeParams>();

        public uint ExecutePeriodMsec => 1000;

        public string Name => "生命周期控制系统";

        public LifetimeParams GetItem(UInt128 RtID)
        {
            lock (this)
            {
                return _items_store[RtID];
            }
        }

        public void RegisterItem(LifetimeParams entity_params)
        {
            lock (this)
            {
                _items_store[entity_params.BindEntity.RuntimeID] = entity_params;
            }
        }

        private void spawn_update(WsSystemHost host, LifetimeParams it, UInt128 current_timepoint)
        {
            var timepoint_birth = it.SpawnStart;
            var growth_span = current_timepoint - timepoint_birth;
            it.LifetimeAcc = growth_span;

            // 成长期不繁育
            if (it.LifetimeAcc <= it.GrowthTimeCount.Value)
                return;

            // 死亡移除
            if (it.LifetimeAcc >= it.DeathTimePoint.Value)
            {
                it.BindEntity.DeathExecute(host);
                return;
            }

            // 记录成熟期初始体积
            if (it.MatureVolume <= 0)
            {
                var vol_sys = host.GetSystem<ShapeVolumeParams, ShapeVolumeSystem>();
                var vol_param = vol_sys.GetItem(it.BindEntity.RuntimeID);
                var birth_volume = vol_param.PrimitiveHeight.Value * vol_param.PrimitiveWidth.Value * vol_param.PrimitiveLength.Value;

                if (vol_param.Volume > birth_volume * 20000)
                {
                    it.MatureVolume = birth_volume * 20000;
                }
                else
                {
                    it.MatureVolume = vol_param.Volume;
                }
            }

            // 成熟期开始生育
            if (it.LifetimeAcc <= it.GrowthTimeCount.Value + it.MatureTimeCount.Value)
            {
                // 执行生育计算
                var span = it.LifetimeAcc - it.GrowthTimeCount.Value;
                if (it.SpawnTimes < (span / it.SpawnPeriods.Value))
                {
                    var vol_sys = host.GetSystem<ShapeVolumeParams, ShapeVolumeSystem>();
                    var vol_param = vol_sys.GetItem(it.BindEntity.RuntimeID);

                    var current_volume = vol_param.Volume;
                    var volume_redundance = current_volume - it.MatureVolume;
                    volume_redundance -= Math.Pow(volume_redundance, 0.75);
                    var birth_volume = vol_param.PrimitiveHeight.Value * vol_param.PrimitiveWidth.Value * vol_param.PrimitiveLength.Value;

                    // 纠正繁育周期错误
                    if (volume_redundance < birth_volume)
                    {
                        var xval = it.SpawnPeriods.Value * 2;
                        it.SpawnPeriods.Value = xval;
                        return;
                    }

                    // 执行繁育
                    it.SpawnTimes++;
                    var spawn_count = (UInt128)(volume_redundance / birth_volume);
                    Console.WriteLine($"繁育时间：time={current_timepoint}");
                    for (UInt128 idx = 0; idx < spawn_count; ++idx)
                    {
                        var new_entity = it.BindEntity.NewEmpty(host.AllocNewID());
                        new_entity.ActiveConfigBeforeExecute(host);
                        current_volume -= birth_volume;
                    }

                    // 校正体积
                    var times = Math.Pow(current_volume / birth_volume, 1 / 3.0);
                    vol_param.Length = times * vol_param.PrimitiveLength.Value;
                    vol_param.Width = times * vol_param.PrimitiveWidth.Value;
                    vol_param.Height = times * vol_param.PrimitiveHeight.Value;
                }
            }
        }
        public void Run(WsSystemHost host, UInt128 timepoint_msec)
        {
            var srcl = _items_store.Values.ToList();
            var opts = new ParallelOptions { MaxDegreeOfParallelism = Environment.ProcessorCount };
            Parallel.For(0, srcl.Count, opts, index =>
            {
                var it = srcl[index];
                spawn_update(host, it, timepoint_msec);
            });
        }

        public void RemoveItem(UInt128 RtID)
        {
            lock (this)
            {
                _items_store.Remove(RtID);
            }
        }
    }
}
