﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SharpWorld
{
    /// <summary>
    /// 可变参数类型共同接口
    /// </summary>
    public interface VariableVal {
        /// <summary>
        /// 参数名称
        /// </summary>
        string Name { get; }
        /// <summary>
        /// 以指定比率执行变异
        /// </summary>
        /// <param name="rates">变异比率</param>
        void Variation(double rates);
    }

    /// <summary>
    /// 参与系统运算的个体数据
    /// </summary>
    public interface WsItemParams {
        /// <summary>
        /// 获取所有可变异参数
        /// </summary>
        List<VariableVal> TotalVariables { get; }

        /// <summary>
        /// 运行时绑定特性演算系统
        /// </summary>
        WsBaseSystem? BindSystem { get; set; }
        /// <summary>
        /// 绑定实体
        /// </summary>
        WsEntity BindEntity { get; }
    }


    /// <summary>
    /// 增强特性运行系统
    /// </summary>
    public interface WsBaseSystem {
        /// <summary>
        /// 系统组件名称
        /// </summary>
        string Name { get; }
    }

    /// <summary>
    /// 可运行系统
    /// </summary>
    public interface WsExecutableSystem : WsBaseSystem
    {
        /// <summary>
        /// 执行周期（以毫秒计）
        /// </summary>
        UInt32 ExecutePeriodMsec { get; }

        /// <summary>
        /// 执行方法
        /// <param name="host">host系统实例</param>
        /// <param name="timepoint_msec">当前仿真时间点，毫秒计</param>
        /// </summary>
        void Run(WsSystemHost host, UInt128 timepoint_msec);
    }

    public interface WsSystem<Datas> : WsBaseSystem 
        where Datas : WsItemParams
    {
        /// <summary>
        /// 在系统内注册实体参数
        /// </summary>
        /// <typeparam name="Datas">参数类型</typeparam>
        /// <param name="entity_params">参数实例</param>
        void RegisterItem(Datas entity_params);

        /// <summary>
        /// 移除实体
        /// </summary>
        /// <param name="RtID"></param>
        void RemoveItem(UInt128 RtID);

        /// <summary>
        /// 通过实体id获取在此系统中绑定的参数包
        /// </summary>
        /// <param name="RtID"></param>
        /// <returns></returns>
        Datas GetItem(UInt128 RtID);
    }

    /// <summary>
    /// 系统访问管理接口
    /// </summary>
    public interface WsSystemHost
    {
        /// <summary>
        /// 下一个实体的id
        /// </summary>
        public UInt128 AllocNewID();

        /// <summary>
        /// 注册可以执行的系统
        /// </summary>
        /// <param name="inst"></param>
        void RegisterSystem<Datas>(WsSystem<Datas> inst) where Datas : WsItemParams;

        /// <summary>
        /// 通过类型获取系统实例
        /// </summary>
        /// <typeparam name="Sys">系统类型</typeparam>
        /// <returns></returns>
        WsSystem<Datas> GetSystem<Datas, Sys>() where Datas : WsItemParams where Sys: WsSystem<Datas>;
    }

    /// <summary>
    /// 实体
    /// </summary>
    public interface WsEntity
    {
        /// <summary>
        /// 创建一个空实例
        /// </summary>
        /// <param name="new_id">实体id</param>
        /// <returns>空实例</returns>
        WsEntity NewEmpty(UInt128 new_id);

        /// <summary>
        /// 实体类型
        /// </summary>
        string TypeName { get; set; }
        /// <summary>
        /// 运行时id
        /// </summary>
        UInt128 RuntimeID { get; set; }

        /// <summary>
        /// 在系统运行前执行动态配置
        /// </summary>
        /// <param name="host"></param>
        void ActiveConfigBeforeExecute(WsSystemHost host);
        void DeathExecute(WsSystemHost host);

        /// <summary>
        /// 存取序列化Json内容
        /// </summary>
        string JsonContent { get; set; }
    }
}
